package com.cosmic_dust.flutter_hyphenate

import android.content.Context
import android.os.Handler
import android.os.Looper
import androidx.annotation.NonNull
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.EventChannel
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry.Registrar


/** FlutterHyphenatePlugin */
class FlutterHyphenatePlugin : FlutterPlugin, MethodCallHandler, EventChannel.StreamHandler {
    /// The MethodChannel that will the communication between Flutter and native Android
    ///
    /// This local reference serves to register the plugin with the Flutter Engine and unregister it
    /// when the Flutter Engine is detached from the Activity
    private lateinit var methodChannel: MethodChannel

    private lateinit var eventChannel: EventChannel

    private val hyphenateClient = HyphenateClient()

    private lateinit var flutterContext: Context

    private var handler = Handler(Looper.getMainLooper())


    override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        flutterContext = flutterPluginBinding.applicationContext
        methodChannel = MethodChannel(flutterPluginBinding.flutterEngine.dartExecutor, "hyphenate_plugin/methods")
        methodChannel.setMethodCallHandler(this)
        eventChannel = EventChannel(flutterPluginBinding.flutterEngine.dartExecutor, "hyphenate_plugin/events")
        eventChannel.setStreamHandler(this)
    }

    // This static function is optional and equivalent to onAttachedToEngine. It supports the old
    // pre-Flutter-1.12 Android projects. You are encouraged to continue supporting
    // plugin registration via this function while apps migrate to use the new Android APIs
    // post-flutter-1.12 via https://flutter.dev/go/android-project-migration.
    //
    // It is encouraged to share logic between onAttachedToEngine and registerWith to keep
    // them functionally equivalent. Only one of onAttachedToEngine or registerWith will be called
    // depending on the user's project. onAttachedToEngine or registerWith must both be defined
    // in the same class.
    companion object {
        var flutterContext: Context? = null
        var activeFlutterContext: Context? = null


        @JvmStatic
        fun registerWith(registrar: Registrar) {
            val methodChannel = MethodChannel(registrar.messenger(), "hyphenate_plugin/methods")
            methodChannel.setMethodCallHandler(FlutterHyphenatePlugin())
            val eventChannel = EventChannel(registrar.messenger(), "hyphenate_plugin/events")
            eventChannel.setStreamHandler(FlutterHyphenatePlugin())
            flutterContext = registrar.context()
            activeFlutterContext = registrar.activeContext()
        }
    }

    override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
        val username: String? = call.argument("username")
        val password: String? = call.argument("password")
        val toChatUsername: String? = call.argument("toChatUsername")
        val isGroupChat = call.argument("isGroupChat") as Boolean?
        when (call.method) {
            "initHyphenate" -> {
                val appKey: String? = call.argument("appKey")
                val isDebug: Boolean? = call.argument("isDebug")
                hyphenateClient.initHyphenate(flutterContext!!, result, appKey!!, false, handler)
            }
            "createAccount" -> {
                hyphenateClient.createAccount(username, password, result, handler)
            }
            "login" -> {
                hyphenateClient.login(username, password, result, handler)
            }
            "logout" -> {
                hyphenateClient.logout(result,handler)
            }
            "sendTxtMsg" -> {
                val content: String? = call.argument("content")
                hyphenateClient.sendTxtMsg(toChatUsername, content, isGroupChat!!, result, handler)
            }

            else -> handler.post { result.notImplemented() }
        }
    }

    override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
        methodChannel.setMethodCallHandler(null)
        eventChannel.setStreamHandler(null)
    }

    override fun onListen(@NonNull arguments: Any, @NonNull events: EventChannel.EventSink) {
        hyphenateClient.addEMMessageListener(events, handler)
    }

    override fun onCancel(arguments: Any?) {
        TODO("Not yet implemented")
    }
}
